package cn.com.zyjblogs.chat.netty;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.logging.LogLevel;
import io.netty.handler.logging.LoggingHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.Closeable;

/**
 * @author Gjing
 **/
@Service
public class ChatServer implements Runnable, Closeable {
    Logger log = LoggerFactory.getLogger(ChatServer.class);
    @Resource
    private ChatServerInitializer chatServerInitializer;

    /**
     * netty服务监听端口
     */
    @Value("${spring.netty.port:8808}")
    private  int port;
    /**
     * 主线程组数量
     */
    @Value("${spring.netty.bossThread:1}")
    private int bossThread;
    private ServerBootstrap serverBootstrap = new ServerBootstrap();
    /**
     * 创建两个线程组，bossGroup为接收请求的线程组，一般1-2个就行
     */
    private NioEventLoopGroup bossGroup = new NioEventLoopGroup(this.bossThread);
    /**
     * 实际工作的线程组
     */
    private NioEventLoopGroup workerGroup = new NioEventLoopGroup();


    /**
     * 初始化netty配置
     */
    private void init() {
        // 两个线程组加入进来
        this.serverBootstrap.group(bossGroup, workerGroup)
                //在bossGroup中添加一个日志处理器
                .handler(new LoggingHandler(LogLevel.INFO))
                // 配置为nio类型
                .channel(NioServerSocketChannel.class)
                // 加入自己的初始化器
                .childHandler(this.chatServerInitializer);
        //对通道进行监听
        this.serverBootstrap.bind(this.port);
        log.info("bind port : {}",this.port);
        log.info("netty server thread finish");
    }
    /**
     * 启动netty服务器
     */
    @Override
    public void run() {
        init();
    }

    @Override
    public void close() {
        //优雅关闭
        log.info("netty closing ...");
        bossGroup.shutdownGracefully();
        workerGroup.shutdownGracefully();
        log.info("netty close");
    }
}
