package cn.com.zyjblogs.chat.netty;

import cn.com.zyjblogs.chat.netty.handler.ChatServerHandler;
import cn.com.zyjblogs.chat.netty.handler.ClientHeartBeatHandler;
import cn.com.zyjblogs.chat.netty.handler.ClientManageHandler;
import cn.com.zyjblogs.chat.netty.po.Msg;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.socket.SocketChannel;
import io.netty.handler.codec.protobuf.ProtobufDecoder;
import io.netty.handler.codec.protobuf.ProtobufEncoder;
import io.netty.handler.timeout.IdleStateHandler;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;


/**
 * Socket 初始化器，每一个Channel进来都会调用这里的 InitChannel 方法
 * @author Gjing
 **/
@Service
public class ChatServerInitializer extends ChannelInitializer<SocketChannel> {
    @Resource
    private ChatServerHandler chatServerHandler;
    @Resource
    private ClientHeartBeatHandler clientHeartBeatHandler;
    @Resource
    private ClientManageHandler clientManageHandler;
    @Override
    protected void initChannel(SocketChannel ch)  {
        ch.pipeline()
                .addLast("decoder",new ProtobufDecoder(Msg.DataContent.getDefaultInstance()))
                /**
                 * IdleStateHandler 是netty 提供的处理空闲状态的处理器
                 * 1、long readerIdleTime ： 表示多长时间没有读，就会发送一个心跳检测包检测是否连接
                 * 2、long writerIdleTime ： 表示多长时间没有写，就会发送一个心跳检测包
                 * 3、long allIdleTime ： 表示多长时间没有读写 就会发送一个心跳检测包
                 * 当IdleStateEvent触发后，就会传递给管道的下一个handler区处理通过调用（触发）
                 * 下一个handler的userEventTiggered，该方法中区处理
                 * IdleStateEvent（读空闲，写空闲，都写空闲）
                 */
                .addLast(new IdleStateHandler(25, 0, 10, TimeUnit.SECONDS))
                .addLast(clientManageHandler)
                //加入一个对空闲检测进一步处理的handler（自定义）
                .addLast(clientHeartBeatHandler)
                .addLast(chatServerHandler)
                .addLast("encoder",new ProtobufEncoder());
    }
}
