package cn.com.zyjblogs.chat.netty.channel;

import com.alibaba.ttl.TransmittableThreadLocal;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class UserChannelContext {
    public UserChannelContext(){

    }
    /**
     * 用户名: ctx
     */
    private static final ThreadLocal<Map<String, Channel>> CONTEXT = new TransmittableThreadLocal<Map<String,Channel>>(){
        @Override
        public Map<String,Channel> initialValue(){
            return new ConcurrentHashMap<>();
        }
    };

    /**
     * 获取CONTEXT
     * @return
     */
    public static synchronized Map<String,Channel> get(){
        return CONTEXT.get();
    }

    /**
     * 设置CONTEXT
     * @param map
     */
    public static synchronized void set(Map<String,Channel> map){
        CONTEXT.set(map);
    }

    /**
     * 新增用户ctx
     * @param username
     * @param ctx
     */
    public static synchronized void putUserCtx(String username,Channel ctx){
        Map<String, Channel> contextMap = get();
        contextMap.put(username,ctx);
        set(contextMap);
    }

    /**
     * 获取ctx
     * @param username
     * @return
     */
    public static synchronized  Channel getUserCtx(String username){
        return get().get(username);
    }

    /**
     * 移除用户ctx
     * @param username
     */
    public static synchronized void removeUserCtx(String username){
        Map<String, Channel> contextMap = get();
        contextMap.remove(username);
        set(contextMap);
    }

    /**
     * 清空
     */
    public static synchronized void clear(){
        CONTEXT.remove();
    }
}
