package cn.com.zyjblogs.chat.netty.handler;

import cn.com.zyjblogs.chat.netty.channel.UserChannelContext;
import cn.com.zyjblogs.chat.netty.factory.MessageHandleFactory;
import cn.com.zyjblogs.chat.netty.factory.strategy.message.MessageHandler;
import cn.com.zyjblogs.chat.netty.po.Msg;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.util.CharsetUtil;
import io.netty.util.ReferenceCountUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;


@Component
@ChannelHandler.Sharable
public class ChatServerHandler extends SimpleChannelInboundHandler<Msg.DataContent> {
    Logger log = LoggerFactory.getLogger(ChatServerHandler.class);
    @Resource
    private MessageHandleFactory factory;

    /**
     * 当通道就绪就会触发该方法
     * @param ctx
     * @param dataContent
     * @throws Exception
     */
    @Override
    protected void channelRead0(ChannelHandlerContext ctx, Msg.DataContent dataContent) throws Exception {
        Msg.DataContent.DataType dataType = dataContent.getDataType();
        switch (dataType.getNumber()){
            case Msg.DataContent.DataType.CONNECTING_VALUE:
                log.info("有客户端连接... " + ctx.channel().remoteAddress().toString());
                //将channel加入到map中
                UserChannelContext.putUserCtx(dataContent.getFromUsername(),ctx.channel());
                break;
            case Msg.DataContent.DataType.CLOSING_VALUE:
                UserChannelContext.removeUserCtx(dataContent.getFromUsername());
                break;
            //聊天逻辑
            case Msg.DataContent.DataType.CHAT_VALUE:
                MessageHandler messageHandler = factory.getMessageContent().loadStrategy(dataType.getNumber());
                messageHandler.contentHandle(dataContent);
                break;
            default:
                break;
        }

        ctx.fireChannelRead(dataContent);
        ctx.fireChannelActive();
        //触发后续调用链
        ReferenceCountUtil.retain(dataContent);
    }

    //数据读取完毕
    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
        // super.channelReadComplete(ctx);
        //writeAndFlush  是write和flush 方法  将数据写入到缓存 并刷新
        //一般将，我们需要对发送数据进行编码
        ctx.writeAndFlush(Unpooled.copiedBuffer("hello,客户端~o( =∩ω∩= )m喵 || 1", CharsetUtil.UTF_8));
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        ctx.close();
    }
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        super.channelActive(ctx);
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        String remoteAddress = ctx.channel().remoteAddress().toString();
        log.info("有客户端断开连接... " + remoteAddress);
        super.channelInactive(ctx);
    }
}
