package cn.com.zyjblogs.chat.netty.handler;

import cn.com.zyjblogs.chat.netty.po.Msg;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.timeout.IdleStateEvent;
import org.springframework.stereotype.Component;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Component
@ChannelHandler.Sharable
public class ClientHeartBeatHandler extends ChannelInboundHandlerAdapter {
    Logger logger = LoggerFactory.getLogger(ClientHeartBeatHandler.class);

    /***
     * 事件
     * @param ctx 上下文
     * @param evt 事件
     * @throws Exception
     */
    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        boolean flag = false;
        if (evt instanceof IdleStateEvent){
            //将evt 向下转型 IdleStateEvent
            IdleStateEvent event = (IdleStateEvent) evt;
            String eventType = null;

            switch (event.state()){
                case READER_IDLE:
                    eventType = "读空闲";
                    flag = true;
                    //如果发生空闲，我们关闭通道
                    ctx.writeAndFlush(Msg.DataContent.newBuilder().setDataType(Msg.DataContent.DataType.CLOSING).build());
                    break;
                case WRITER_IDLE:
                    eventType = "写空闲";
                    break;
                case ALL_IDLE:
                    eventType = "读写空闲";
                    //如果发生空闲，我们关闭通道
                    ctx.writeAndFlush(Msg.DataContent.newBuilder().setDataType(Msg.DataContent.DataType.KEEPALIVE).build());
                    break;
                default:
                    eventType = "类型未识别";
                    break;
            }
            logger.info("{}",ctx.channel().remoteAddress()+"--TimeOut Type--"+eventType);
        }
        if (flag){
            ctx.close();
        } else {
            super.userEventTriggered(ctx, evt);
        }
    }
}
